@@ -19,6 +19,7 @@ Usage: $0
 
 Options:
   -h, --help        show this help and quit
+  -v, --update-venv install/update all required packages in venv
   -t, --test        run the automated tests
   -m, --merge       set the version number to new release,
                     merge the current development branch into the master
@@ -34,10 +35,11 @@ Options:
 
 # ---------- command line arguments ----------
 
-PARSED_ARGUMENTS=$(getopt -n "$0" -o htmub --long help,test,merge,patch,minor,major,upload,branch -- "$@")
+PARSED_ARGUMENTS=$(getopt -n "$0" -o hvtmub --long help,update-venv,test,merge,patch,minor,major,upload,branch -- "$@")
 
 increment=minor
 
+update_venv=none
 run_tests=none
 merge_and_tag=none
 build_and_upload=none
@@ -47,6 +49,7 @@ eval set -- "$PARSED_ARGUMENTS"
 while true; do
 	case "$1" in
 		-h | --help)        usage; exit ;;
+		-v | --update-venv) update_venv=true;        shift ;;
 		-t | --test)        run_tests=true;          shift ;;
 		-m | --merge)       merge_and_tag=true;      shift ;;
 		     --patch)       increment=patch;         shift ;;
@@ -119,6 +122,9 @@ increment_version() {
 		echo >&2 "Invalid value for increment: '$increment'. Should be one of patch, minor, major"
 		exit 1
 	fi
+	if [ "$oldversion" = '' ]; then
+		oldversion='v0.0.0'
+	fi
 	echo "$oldversion" | perl -na -F'\.' -e "$inccmd"'; print join ".", @F[0..$#F]'
 }
 
@@ -221,39 +227,46 @@ commit_version() {
 }
 
 
-# ---------- download packages for offline usage ----------
-
-package_dir=.package-store
-
-if ! [ -e "$package_dir" ]; then
-	echo $color_cmd"downloading dependencies"$color_reset
-	mkdir "$package_dir"
-	pip download -r requirements.txt -r requirements-release.txt -r requirements-test.txt -d "$package_dir"
-fi
-
-
 # ---------- venv ----------
 
+install_requirements() {
+	echo $color_cmd"installing dependencies"$color_reset
+	python3 -m pip install --upgrade pip
+	python3 -m pip install --upgrade -r requirements.txt
+	python3 -m pip install --upgrade -r requirements-release.txt
+	python3 -m pip install --upgrade -r requirements-test.txt
+	python3 -m pip install -e .
+}
+
 if ! python3 -m flit --version >/dev/null; then
 	if [ ! -d venv ]; then
 		echo $color_cmd"creating new venv"$color_reset
 		python3 -m venv venv
 		. venv/bin/activate
-		python3 -m pip install --upgrade pip
-		python3 -m pip install -r requirements.txt
-		python3 -m pip install -r requirements-release.txt
-		python3 -m pip install -r requirements-test.txt
-		python3 -m pip install -e .
+		install_requirements
 	else
 		echo $color_cmd"activating venv"$color_reset
 		. venv/bin/activate
+		[ "$update_venv" = 'true' ] && install_requirements
 	fi
+else
+	[ "$update_venv" = 'true' ] && install_requirements
+fi
+
+if [ "$update_venv" = 'true' ]; then
+	exit
 fi
 
 
 # ---------- run tests ----------
 
-tox
+tox_exit_code=0
+tox || tox_exit_code=$?
+
+if [ "$tox_exit_code" != "0" ]; then
+	exit $tox_exit_code
+fi
+
 
 if [ "$test_only" = "true" ]; then
 	exit
@@ -263,7 +276,7 @@ fi
 # ---------- main ----------
 
 fn_tag="tag"
-fn_version="src/udsync/main.py"
+fn_version="src/udsync/about.py"
 
 branchname="$(git branch | sed -En 's/^\* (.*)$/\1/p')"
 lastversion="$(git tag --list 'v*' --sort=version:refname | tail -1)"
