@@ -213,8 +213,7 @@ class SubprocessException(Exception):
 
 class Lsblk:
 
-	device_paths_to_be_ignored: typing.ClassVar = Config('devices-to-be-ignored', ['/dev/sda'],
-		help='in the wizard to create a new backup plan these devices are not displayed when selecting the source/destination path')
+	device_paths_to_be_ignored: 'typing.ClassVar[Config[list[str]]]'
 	file_system_types_to_be_ignored = Config('file-system-types-to-be-ignored', [FILESYSTEM_TYPE_SWAP, FILESYSTEM_TYPE_LVM2],
 		help='in the wizard to create a new backup plan filesystems with these types are not displayed when selecting the source/destination path')
 
@@ -234,7 +233,8 @@ class Lsblk:
 			raise SubprocessException(e)
 		return p.stdout.splitlines()
 
-	def iter_all_devices(self, *paths: str, nodeps: bool = False) -> typing.Iterator[Device]:
+	@classmethod
+	def iter_all_devices(cls, *paths: str, nodeps: bool = False) -> typing.Iterator[Device]:
 		cmd = ['lsblk', '-n', '--pairs', '-o', 'PATH,FSTYPE,UUID,MOUNTPOINT,TYPE,LABEL,PARTLABEL,SIZE,VENDOR,MODEL']
 		if nodeps:
 			cmd.append('--nodeps')
@@ -244,16 +244,17 @@ class Lsblk:
 		except (FileNotFoundError, subprocess.CalledProcessError) as e:
 			raise SubprocessException(e)
 		for ln in p.stdout.splitlines():
-			args = dict(self.split_pair(v) for v in shlex.split(ln))
+			args = dict(cls.split_pair(v) for v in shlex.split(ln))
 			yield Device(**args)
 
-	def iter_devices(self, *paths: str) -> typing.Iterator[Device]:
+	@classmethod
+	def iter_devices(cls, *paths: str) -> typing.Iterator[Device]:
 		# I have seen a disk and a partition on it having the same values (UUID, FSTYPE, LABEL)
 		# on a Debian and Arch install stick where the partition was mountable but the disk was not.
 		# Therefore I am ignoring all disks which have the same UUID like a partition.
 		devices: typing.Dict[str, Device] = {}
 		devices_without_uuid = []
-		for dev in self.iter_all_devices(*paths):
+		for dev in cls.iter_all_devices(*paths):
 			if not dev.uuid:
 				devices_without_uuid.append(dev)
 			elif dev.uuid in devices:
@@ -297,6 +298,24 @@ class Lsblk:
 			if not dev.is_mounted():
 				yield dev
 
+	@classmethod
+	def get_root_device_path(cls) -> str:
+		paths = []
+		for dev in cls.iter_all_devices():
+			if dev.mountpoint == '/boot':
+				out = dev.path[:-1]
+				if out.endswith('p'):
+					out = out[:-1]
+				return out
+			paths.append(dev.path)
+		for p in ('/dev/nvme0n1',):
+			if p in paths:
+				return p
+		return '/dev/sda'
+
+Lsblk.device_paths_to_be_ignored = Config('devices-to-be-ignored', [Lsblk.get_root_device_path()],
+	help='in the wizard to create a new backup plan these devices are not displayed when selecting the source/destination path')
+
 
 class Udisksctl:
 
