@@ -1,4 +1,4 @@
-#!/usr/bin/env python3
+#!./runmodule.sh
 
 import os
 import enum
@@ -6,11 +6,11 @@ import abc
 import typing
 
 import urwid
+from confattr import Config, UiNotifier
+
 from . import urwid_dialog
 from . import urwid_multi_key_support
-
 from .lsblk import lsblk, mounter, Device, SubprocessException
-from .config import Config
 from .urwid_colors import ColorConfig
 
 HelpItem = urwid_multi_key_support.HelpItem
@@ -102,7 +102,6 @@ class ExtendedEdit(urwid.WidgetWrap):
 			self.set_edit_text('')
 			self.set_edit_pos(0)
 		else:
-			assert cmd not in self._command_map.implemented_commands
 			return key
 
 		assert cmd in self._command_map.implemented_commands
@@ -112,6 +111,10 @@ class ExtendedEdit(urwid.WidgetWrap):
 		self.edit_attr_map.set_attr_map({None:self.color_invalid})
 		self.edit_attr_map.set_focus_map({None:self.color_invalid_focus})
 
+	def valid(self) -> None:
+		self.edit_attr_map.set_attr_map({None:self.color_normal})
+		self.edit_attr_map.set_focus_map({None:self.color_normal_focus})
+
 	def __getattr__(self, name: str) -> typing.Any:
 		return getattr(self.edit, name)
 
@@ -168,6 +171,7 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 		key_handler: typing.Optional[typing.Callable[[urwid_multi_key_support.SubCommandMap, URWID_TYPE_SIZE, URWID_TYPE_KEY], typing.Optional[URWID_TYPE_KEY]]] = None,
 	) -> None:
 		self.app = app
+		self.ui_notifier = app.get_ui_notifier()
 		self.show_hidden = show_hidden
 		self.name_of_last_focused_directory: typing.Dict[str, str] = {}
 		self.cancel = cancel
@@ -179,10 +183,10 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 		self.init_path(start_path)
 
 		self.listbox.keypress = self.keypress_listbox
-		self.text_path.keypress = self.keypress_path  # type: ignore [assignment]  # This is a bug in mypy https://github.com/python/mypy/issues/2427
+		self.text_path.keypress = self.keypress_path  # type: ignore [method-assign]  # This is a bug in mypy https://github.com/python/mypy/issues/2427
 
-		self.help_bar = urwid_multi_key_support.HelpBar(self.help_bar_content, MenuWidget._command_map, self.app, edit_context=False)
-		self.help_bar_menu = urwid_multi_key_support.HelpBar(self.help_bar_content_menu, MenuWidget._command_map, self.app, edit_context=False)
+		self.help_bar = urwid_multi_key_support.HelpBar(self.help_bar_content, MenuWidget._command_map, self.ui_notifier, edit_context=False)
+		self.help_bar_menu = urwid_multi_key_support.HelpBar(self.help_bar_content_menu, MenuWidget._command_map, self.ui_notifier, edit_context=False)
 
 		btn_select = urwid_dialog.ColorButton(self.BTN_SELECT, lambda btn: select(self.text_path.text))
 		btn_cancel = urwid_dialog.ColorButton(self.BTN_CANCEL, lambda btn: cancel())
@@ -215,7 +219,7 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 			try:
 				dev = mounter.mount_path_if_necessary(start_path, self.app.password_screen())
 			except SubprocessException as e:
-				self.app.show_error(e)
+				self.ui_notifier.show_error(e)
 				self.set_path('')
 				return
 
@@ -307,7 +311,7 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 			directory_names.sort()
 		else:
 			directory_names = []
-			self.app.show_error('Directory does not exist: %r' % path)
+			self.ui_notifier.show_error('Directory does not exist: %r' % path)
 		directory_names.insert(0, os.path.pardir)
 		return [DirectoryWidget(self, uuid_based_path, dn) for dn in directory_names]
 
@@ -319,7 +323,7 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 		try:
 			devices = list(lsblk.iter_interesting_devices())
 		except SubprocessException as e:
-			self.app.show_error(e)
+			self.ui_notifier.show_error(e)
 			devices = []
 		mounted_devices = [dev for dev in devices if dev.is_mounted()]
 		unmounted_devices = [dev for dev in devices if not dev.is_mounted()]
@@ -534,7 +538,7 @@ class MenuWidget(urwid.WidgetWrap):
 				with self.app.password_screen():
 					dev = self.device.unlock()
 			except SubprocessException as exc:
-				self.app.show_error(exc)
+				self.ui_notifier.show_error(exc)
 				return None
 		else:
 			dev = self.device
@@ -555,7 +559,7 @@ class MenuWidget(urwid.WidgetWrap):
 				return backing_device
 
 		except SubprocessException as e:
-			self.app.show_error(e)
+			self.ui_notifier.show_error(e)
 
 		return self.device
 
@@ -603,12 +607,24 @@ class PathEdit(urwid.WidgetWrap):
 		self.app.open_view(self.last_view)
 
 
-	def validate(self) -> bool:
-		if mounter.is_valid_path(self.get_path(), self.app.password_screen()):
-			return True
+	def get_invalid_path(self) -> 'list[tuple[str, bool]]':
+		'''
+		:return: list of one or zero (expanded path, can be created) tuples, empty list means the path is an existing directory
+		'''
+		path = self.get_path()
+		password_entry_context = self.app.password_screen()
+		try:
+			path = mounter.expand_path(path, password_entry_context)
+		except SubprocessException:
+			self.edit.invalid()
+			return [(path, False)]
 
-		self.edit.invalid()
-		return False
+		if not os.path.isdir(path):
+			self.edit.invalid()
+			return [(path, True)]
+
+		self.edit.valid()
+		return []
 
 	def get_path(self) -> str:
 		return typing.cast(str, self.edit.get_edit_text())
@@ -640,6 +656,14 @@ if __name__ == '__main__':
 		raise urwid.ExitMainLoop()
 
 	class OverlayOpener(App):
+
+		def __init__(self) -> None:
+			self.ui_notifier = UiNotifier()
+			self.ui_notifier.set_ui_callback(print)
+
+		def get_ui_notifier(self) -> UiNotifier:
+			return self.ui_notifier
+
 		def open_pressed_keys_overlay(self, keys: typing.Sequence[URWID_TYPE_KEY], command_map: urwid_multi_key_support.SubCommandMap) -> None:
 			self._prev_widget = loop.widget
 			loop.widget = urwid_multi_key_support.OverlayPressedKeys(self._prev_widget, keys, command_map)
@@ -661,11 +685,6 @@ if __name__ == '__main__':
 		def show_help_bar(self, help_bar: urwid_multi_key_support.HelpBar) -> None:
 			pass
 
-		def show_error(self, msg: typing.Union[str, BaseException]) -> None:
-			print(msg)
-		def show_info(self, msg: str) -> None:
-			print(msg)
-
 	loop = urwid.MainLoop(DirectoryChooser(OverlayOpener(), on_select, on_cancel))
 	ColorConfig.set_register_color(lambda c: typing.cast(None, loop.screen.register_palette_entry(*c.to_palette_tuple())))
 	loop.run()
