@@ -1,4 +1,4 @@
-#!/usr/bin/env python3
+#!./runmodule.sh
 
 import re
 import shlex
@@ -10,10 +10,14 @@ import urwid
 
 from gettext import gettext as _
 
-from .base_classes import Logger
-from .config import Config
+from confattr import Config, ConfigFile, ConfigFileCommand, ConfigFileArgparseCommand, FormattedWriter, SectionLevel, UiNotifier, NotificationLevel
+
 from .urwid_colors import ColorStr, ColorConfig, URWID_TYPE_MARKUP
 
+if typing.TYPE_CHECKING:
+	from typing_extensions import Unpack
+	from confattr import SaveKwargs
+
 URWID_TYPE_SIZE = typing.Union[typing.Tuple[int], typing.Tuple[int, int]]
 URWID_TYPE_KEY = str
 URWID_TYPE_WIDGET = typing.Any
@@ -29,8 +33,8 @@ class KeyMapper:
 
 	# ------- init -------
 
-	def init_key_mapper(self, logger: Logger, command_map: 'SubCommandMap') -> None:
-		self.logger = logger
+	def init_key_mapper(self, ui_notifier: UiNotifier, command_map: 'SubCommandMap') -> None:
+		self.logger = ui_notifier
 		self._default_command_map = command_map
 
 	# ------- splitting and joining keys -------
@@ -227,7 +231,7 @@ class MultiKeySupport(KeyMapper):
 	_command_map: 'SubCommandMap'
 
 	def init_multi_key_support(self, overlay_opener: 'OverlayPressedKeysOpener') -> None:
-		self.init_key_mapper(overlay_opener, self._command_map)
+		self.init_key_mapper(overlay_opener.get_ui_notifier(), self._command_map)
 		self._pressed_keys: typing.List[URWID_TYPE_KEY] = []
 		self._overlay_opener = overlay_opener
 
@@ -312,144 +316,45 @@ def replace_command(command_map: SubCommandMap, old_cmd: str, new_cmd: str) -> N
 
 # ========== config file ==========
 
+class UrwidConfigFileArgparseCommand(ConfigFileArgparseCommand, abstract=True):
 
-class ParseException(Exception):
-	pass
-
-class ArgumentParser(argparse.ArgumentParser):
-
-	def error(self, message: str) -> typing.NoReturn:
-		raise ParseException(message)
+	def init_parser(self, parser: argparse.ArgumentParser) -> None:
+		assert isinstance(self.config_file, UrwidConfigFile)
+		self.urwid_config_file = self.config_file
 
 
-class CommandMapExporter:
+class Map(UrwidConfigFileArgparseCommand):
 
-	COMMENT_PREFIXES = ('"', '#')
+	def init_parser(self, parser: argparse.ArgumentParser) -> None:
+		super().init_parser(parser)
+		parser.add_argument('maps', nargs='?', type=KeyMapName(self.urwid_config_file.keymaps), default=KeyMapName.ALL)
+		parser.add_argument('key')
+		parser.add_argument('cmd')
 
-	MAP = ('map',)
-	UNMAP = ('unmap', 'unm')
-	CLEAR = ('mapclear', 'mapc')
-
-	def __init__(self,
-		logger: Logger,
-		command_maps: typing.Union[typing.Dict[str, SubCommandMap], SubCommandMap],
-		reference_command_map: SubCommandMap = SubCommandMap(),
-	) -> None:
-		self.logger = logger
-		if not isinstance(command_maps, dict):
-			command_maps = {'general' : command_maps}
-		self.keymaps = {name:KeyMapper() for name in command_maps}
-		for name in command_maps:
-			self.keymaps[name].init_key_mapper(logger, command_maps[name])
-		self.reference_command_map = reference_command_map
-
-	# ------- load -------
-
-	def load(self, fn: str) -> None:
-		with open(fn, 'rt') as f:
-			for lnno, ln in enumerate(f, 1):
-				self.parse_line(ln, lnno, f.name)
-
-	def parse_line(self, ln: str, lnno: typing.Optional[int] = None, fn: typing.Optional[str] = None) -> None:
-		'''
-		:param ln: The line to be parsed
-		:param lnno: The number of the line, used in error messages
-		:param fn: The full name of the file from which ln was read, used in error messages
-
-		Calls :meth:`parse_error` if something goes wrong, e.g. too many arguments
-		'''
-		ln = ln.strip()
-		if not ln:
-			return
-		if self.is_comment(ln):
-			return
-
-		ln_splitted = shlex.split(ln, comments=True)
-		self.parse_splitted_line(ln_splitted, ln, lnno, fn)
-
-	def parse_splitted_line(self, ln_splitted: typing.Sequence[str], ln: str, lnno: typing.Optional[int] = None, fn: typing.Optional[str] = None) -> None:
-		cmd = ln_splitted[0]
-
-		try:
-			if cmd in self.MAP:
-				self.map(ln_splitted)
-			elif cmd in self.UNMAP:
-				self.unmap(ln_splitted)
-			elif cmd in self.CLEAR:
-				self.mapclear(ln_splitted)
-			else:
-				self.unknown_command(ln_splitted, ln, lnno, fn)
-		except ParseException as e:
-			self.parse_error(str(e), ln, lnno)
-			return
-
-	def is_comment(self, ln: str) -> bool:
-		for c in self.COMMENT_PREFIXES:
-			if ln.startswith(c):
-				return True
-		return False
-
-
-	def map(self, cmd: typing.Sequence[str]) -> None:
-		p = ArgumentParser(cmd[0])
-		p.add_argument('maps', nargs='?', type=KeyMapName(self.keymaps), default=KeyMapName.ALL)
-		p.add_argument('key')
-		p.add_argument('cmd')
-
-		args = p.parse_args(cmd[1:])
+	def run_parsed(self, args: argparse.Namespace) -> None:
 		for map_name in args.maps:
-			self.keymaps[map_name].bind_key(args.key, args.cmd)
+			self.urwid_config_file.keymaps[map_name].bind_key(args.key, args.cmd)
 
-	def unmap(self, cmd: typing.Sequence[str]) -> None:
-		p = ArgumentParser(cmd[0])
-		p.add_argument('maps', nargs='?', type=KeyMapName(self.keymaps), default=KeyMapName.ALL)
-		p.add_argument('key')
+	def save(self, writer: FormattedWriter, **kw: 'Unpack[SaveKwargs]') -> None:
+		if self.should_write_heading:
+			writer.write_heading(SectionLevel.SECTION, 'Key bindings')
 
-		args = p.parse_args(cmd[1:])
-		for map_name in args.maps:
-			self.keymaps[map_name].unbind_key(args.key)
+		clear_pattern = Mapclear.get_name()
+		map_pattern = Map.get_name() + ' {key} {cmd}'
+		specific_map_pattern = Map.get_name() + ' {map_name} {key} {cmd}'
+		specific_unmap_pattern = Unmap.get_name() + ' {map_name} {key}'
 
-	def mapclear(self, cmd: typing.Sequence[str]) -> None:
-		p = ArgumentParser(cmd[0])
-		p.add_argument('maps', nargs='?', type=KeyMapName(self.keymaps), default=KeyMapName.ALL)
+		writer.write_command(clear_pattern)
+		writer.write_line('')
 
-		args = p.parse_args(cmd[1:])
-		for map_name in args.maps:
-			self.keymaps[map_name].mapclear()
-
-	def unknown_command(self, ln_splitted: typing.Sequence[str], ln: str, lnno: typing.Optional[int], fn: typing.Optional[str]) -> typing.Optional[typing.NoReturn]:
-		raise ParseException('unkown command %r' % ln_splitted[0])
-
-
-	# ------- save -------
-
-	def save(self, fn: str) -> None:
-		'''
-		Save the current key mappings to a file.
-
-		:param fn: The name of the file to write to
-		'''
-		with open(fn, 'wt') as f:
-			self.save_to_open_file(f)
-
-	def save_to_open_file(self, f: typing.TextIO) -> None:
-		clear_pattern = self.CLEAR[0] + '\n'
-		map_pattern = self.MAP[0] + ' {key}\t{cmd}\n'
-		specific_map_pattern = self.MAP[0] + ' {map_name} {key}\t{cmd}\n'
-		specific_unmap_pattern = self.UNMAP[0] + ' {map_name} {key}\n'
-		n = '\n'
-
-		f.write(clear_pattern)
-
-		keys_defined_for_all = dict(KeyMapper.iter_commands_unsorted(self.reference_command_map))
+		keys_defined_for_all = dict(KeyMapper.iter_commands_unsorted(self.urwid_config_file.reference_command_map))
 		keys_defined_for_all.update(self.keys_with_same_commands())
 		for key, cmd in sorted(keys_defined_for_all.items(), key=KeyMapper.sortkey_key_cmd):
 			ln = map_pattern.format(key=shlex.quote(key), cmd=shlex.quote(cmd))
-			f.write(n + ln)
-			n = ''
+			writer.write_command(ln)
 
-		for map_name, keymapper in self.keymaps.items():
-			n = '\n'
+		for map_name, keymapper in self.urwid_config_file.keymaps.items():
+			linebreak = True
 			keys_to_be_unbound = list(keys_defined_for_all.keys())
 			for key, cmd in keymapper.iter_commands(keymapper._default_command_map):
 				try:
@@ -458,17 +363,21 @@ class CommandMapExporter:
 					pass
 				if keys_defined_for_all.get(key, None) == cmd:
 					continue
+				if linebreak:
+					writer.write_line('')
+					linebreak = False
 				ln = specific_map_pattern.format(map_name=map_name, key=shlex.quote(key), cmd=shlex.quote(cmd))
-				f.write(n + ln)
-				n = ''
+				writer.write_command(ln)
 
 			for key in keys_to_be_unbound:
+				if linebreak:
+					writer.write_line('')
+					linebreak = False
 				ln = specific_unmap_pattern.format(map_name=map_name, key=shlex.quote(key))
-				f.write(n + ln)
-				n = ''
+				writer.write_command(ln)
 
 	def keys_with_same_commands(self) -> typing.Mapping[str, str]:
-		s_s_key_cmd = [tuple(keymapper.iter_commands(keymapper._default_command_map)) for keymapper in self.keymaps.values()]
+		s_s_key_cmd = [tuple(keymapper.iter_commands(keymapper._default_command_map)) for keymapper in self.urwid_config_file.keymaps.values()]
 		keys_defined_in_all_maps = set.intersection(*(set(key for key, cmd in s_key_cmd) for s_key_cmd in s_s_key_cmd))
 
 		keys_equal_in_all_maps = {key:cmd for key, cmd in s_s_key_cmd[0] if key in keys_defined_in_all_maps}
@@ -480,24 +389,51 @@ class CommandMapExporter:
 
 		return keys_equal_in_all_maps
 
+class Unmap(UrwidConfigFileArgparseCommand):
 
-	# ------- error handling -------
+	aliases = ('unm',)
 
-	def parse_error(self, msg: str, ln: str, lnno: typing.Optional[int]) -> None:
-		'''
-		Is called if something went wrong while trying to load a key mappings file.
-		This method compiles the given information into an error message and calls :meth:`Logger.show_error`.
+	def init_parser(self, parser: argparse.ArgumentParser) -> None:
+		super().init_parser(parser)
+		parser.add_argument('maps', nargs='?', type=KeyMapName(self.urwid_config_file.keymaps), default=KeyMapName.ALL)
+		parser.add_argument('key')
 
-		:param msg: The error message
-		:param ln: The line where the error occured
-		:param lnno: The number of the line
-		'''
-		if lnno is not None:
-			lnref = 'line %s' % lnno
-		else:
-			lnref = 'line'
-		msg +=  f' while trying to parse {lnref} {ln!r}'
-		self.logger.show_error(msg)
+	def run_parsed(self, args: argparse.Namespace) -> None:
+		for map_name in args.maps:
+			self.urwid_config_file.keymaps[map_name].unbind_key(args.key)
+
+class Mapclear(UrwidConfigFileArgparseCommand):
+
+	aliases = ('mapc',)
+
+	def init_parser(self, parser: argparse.ArgumentParser) -> None:
+		super().init_parser(parser)
+		parser.add_argument('maps', nargs='?', type=KeyMapName(self.urwid_config_file.keymaps), default=KeyMapName.ALL)
+
+	def run_parsed(self, args: argparse.Namespace) -> None:
+		for map_name in args.maps:
+			self.urwid_config_file.keymaps[map_name].mapclear()
+
+
+class UrwidConfigFile(ConfigFile):
+
+	def __init__(self, *,
+		appname: str,
+		command_maps: typing.Union[typing.Dict[str, SubCommandMap], SubCommandMap],
+		reference_command_map: SubCommandMap = SubCommandMap(),
+		notification_level: Config[NotificationLevel],
+		commands: 'typing.Sequence[type[ConfigFileCommand]|abc.ABCMeta]|None' = None,
+	) -> None:
+		if not isinstance(command_maps, dict):
+			command_maps = {'general' : command_maps}
+		self.keymaps = {name:KeyMapper() for name in command_maps}
+		self.reference_command_map = reference_command_map
+
+		# super must be called after keymaps has been set because it initializes the commands which need the names of the keymaps
+		super().__init__(appname=appname, notification_level=notification_level, commands=commands)
+
+		for name in command_maps:
+			self.keymaps[name].init_key_mapper(self.ui_notifier, command_maps[name])
 
 class KeyMapName:
 
@@ -523,7 +459,7 @@ class KeyMapName:
 
 # ========== overlay widget ==========
 
-class OverlayPressedKeysOpener(Logger):
+class OverlayPressedKeysOpener:
 
 	@abc.abstractmethod
 	def open_pressed_keys_overlay(self, keys: typing.Sequence[URWID_TYPE_KEY], command_map: SubCommandMap) -> None:
@@ -533,6 +469,10 @@ class OverlayPressedKeysOpener(Logger):
 	def close_pressed_keys_overlay(self) -> None:
 		raise NotImplementedError()
 
+	@abc.abstractmethod
+	def get_ui_notifier(self) -> UiNotifier:
+		raise NotImplementedError()
+
 
 class OverlayPressedKeys(urwid.Overlay):
 
@@ -553,8 +493,7 @@ class OverlayPressedKeys(urwid.Overlay):
 
 	def __init__(self, bg_widget: urwid.Widget, pressed_keys: typing.Sequence[URWID_TYPE_KEY], command_map: SubCommandMap, *, title: typing.Optional[str] = None) -> None:
 		self.bg_widget = bg_widget
-		# I can set the OverlayPressedKeysOpener to None because the listbox does not receive any keypresses, this widget forwards them all to bg_widget
-		self.listbox = PressedKeysWidget(None, pressed_keys, command_map, title=title)  # type: ignore [arg-type]
+		self.listbox = PressedKeysWidget(None, pressed_keys, command_map, title=title)
 
 		header = urwid.Text('')
 		self.frame = urwid.Frame(self.listbox, header=header)
@@ -595,13 +534,15 @@ class ExtendedListBox(urwid.ListBox, MultiKeySupport):
 	color_title = ColorConfig('help.color.list.title', 'default,bold')
 	color_ellipsis = ColorConfig('help.color.list.ellipsis', 'default')
 
-	def __init__(self, pressed_keys_opener: OverlayPressedKeysOpener, widgets: typing.Sequence[urwid.Widget]) -> None:
+	def __init__(self, pressed_keys_opener: 'OverlayPressedKeysOpener|None', widgets: typing.Sequence[urwid.Widget]) -> None:
+		# pressed_keys_opener is allowed to be None if this widget is used in an overlay which does not receive any keypresses.
 		self.ellipsis_widget = urwid.Text((self.color_ellipsis, self.ELLIPSIS), align=urwid.CENTER)
 		self.key_handler: typing.Optional[typing.Callable[[SubCommandMap, URWID_TYPE_SIZE, URWID_TYPE_KEY], typing.Optional[URWID_TYPE_KEY]]] = None
 
 		body = urwid.SimpleFocusListWalker(widgets)
 		super().__init__(body)
-		self.init_multi_key_support(pressed_keys_opener)
+		if pressed_keys_opener:
+			self.init_multi_key_support(pressed_keys_opener)
 
 	def set_key_handler(self, callback: typing.Callable[[SubCommandMap, URWID_TYPE_SIZE, URWID_TYPE_KEY], typing.Optional[URWID_TYPE_KEY]]) -> None:
 		self.key_handler = callback
@@ -663,7 +604,9 @@ class PressedKeysWidget(ExtendedListBox):
 
 	MAX_WIDTH = 40
 
-	def __init__(self, pressed_keys_opener: OverlayPressedKeysOpener, pressed_keys: typing.Sequence[URWID_TYPE_KEY], command_map: 'SubCommandMap', *, title: typing.Optional[str] = None) -> None:
+	def __init__(self, pressed_keys_opener: 'OverlayPressedKeysOpener|None', pressed_keys: typing.Sequence[URWID_TYPE_KEY], command_map: 'SubCommandMap', *, title: typing.Optional[str] = None) -> None:
+		# pressed_keys_opener can be None because this widget is supposed to be the widget which is displayed by the pressed_keys_opener.
+		# But this widget is also used in the help and there a pressed_keys_opener must be passed.
 		widgets = []
 		if title:
 			widgets.append(urwid.Text((self.color_title, title)))
@@ -743,6 +686,7 @@ class HelpItem:
 
 	type_name = "'help text: command' or 'help text: <key>'"
 	type_article = None
+	help = ''
 
 	SEP = ': '
 	CMD_SEP = '||'
@@ -796,7 +740,7 @@ class HelpBar(urwid.WidgetWrap):
 		help='the separator which is placed between two key mappings for the same command when replacing the {{key}} wildcard in %help-bar.fmt%')
 	unmapped_commands_are_errors = Config('help-bar.show-error-for-unmapped-commands', True)
 
-	def __init__(self, content: typing.Sequence[HelpItem], command_map: SubCommandMap, logger: Logger, *, edit_context: bool) -> None:
+	def __init__(self, content: typing.Sequence[HelpItem], command_map: SubCommandMap, logger: UiNotifier, *, edit_context: bool) -> None:
 		'''
 		the arguments are passed through to :meth:`set_help`
 		'''
