@@ -503,3 +503,21 @@ def test_save_and_load_color_str(fn_config: str) -> None:
 
 	config_exporter.load_file(fn_config)
 	assert t.fmt == '<color=green>hello</color> <color=blue>world</color>'
+
+def test_save_and_load_color_str_with_newline(fn_config: str) -> None:
+	class MyTestClass:
+		fmt = Config('fmt', ColorStr('hello\nworld'))
+
+	t = MyTestClass()
+	assert t.fmt == 'hello\nworld'
+
+	t.fmt = ColorStr('<color=green>hello</color>\n<color=blue>world</color>')
+	config_exporter = create_config_file()
+	config_exporter.save_file(fn_config)
+	assert t.fmt == '<color=green>hello</color>\n<color=blue>world</color>'
+
+	t.fmt = ColorStr('123')
+	assert t.fmt == '123'
+
+	config_exporter.load_file(fn_config)
+	assert t.fmt == '<color=green>hello</color>\n<color=blue>world</color>'
