@@ -1,45 +1,67 @@
-#!/usr/bin/env pytest -s
+#!../venv/bin/pytest -s -vv
 
 import os
 import shutil
+import abc
 import typing
+from collections.abc import Sequence
+
+from confattr import Config, UiNotifier, NotificationLevel, ConfigFileCommand
+import confattr.state
 
 import pytest
+from defs import ui_callback
 
 from udsync import urwid_multi_key_support as sut
-from udsync.base_classes import Logger
 
 import urwid
 urwid.command_map['h'] = urwid.CURSOR_LEFT
 urwid.command_map['l'] = urwid.CURSOR_RIGHT
 
-
 PATH_ROOT = 'autotest'
 FN_CONFIG = os.path.join(PATH_ROOT, 'key-mappings')
 
+if typing.TYPE_CHECKING:
+	from typing_extensions import Unpack, NotRequired
+
+	class CreateConfigFileParams(typing.TypedDict):
+		command_maps: 'NotRequired[dict[str, sut.SubCommandMap]|sut.SubCommandMap]'
+		reference_command_map: 'NotRequired[sut.SubCommandMap]'
+
+
+
 @pytest.fixture()
 def create_test_dir() -> None:
 	if os.path.exists(PATH_ROOT):
 		shutil.rmtree(PATH_ROOT)
 	os.mkdir(PATH_ROOT)
 
+@pytest.fixture()
+def reset_config() -> None:
+	Config.instances.clear()
+	confattr.state.has_config_file_been_instantiated = False
+	confattr.state.has_any_config_file_been_instantiated = False
 
-class ParseError(ValueError):
-	pass
-
-class TestLogger(Logger):
-	def show_error(self, msg: typing.Union[BaseException, str]) -> None:
-		raise ParseError(msg)
-	def show_info(self, msg: str) -> None:
-		pass
+def create_config_file(*, commands: 'Sequence[type[ConfigFileCommand]|abc.ABCMeta]|None' = None, **kw: 'Unpack[CreateConfigFileParams]') -> sut.UrwidConfigFile:
+	kw.setdefault('command_maps', sut.SubCommandMap())
+	out = sut.UrwidConfigFile(
+		appname = 'test',
+		notification_level = NotificationLevel.ERROR,   # type: ignore [arg-type]  # I haven't typed this possibility because I don't want to use this in production code
+		commands = commands,
+		**kw,
+	)
+	out.set_ui_callback(ui_callback)
+	return out
 
 
 class SutKeyMapper(sut.KeyMapper):
 
-	def __init__(self, command_map: typing.Optional[sut.SubCommandMap] = None):
+	def __init__(self, command_map: 'sut.SubCommandMap|None' = None):
 		if command_map is None:
 			command_map = urwid.command_map.copy()
-		self.init_key_mapper(TestLogger(), command_map)
+		ui_notifier = UiNotifier()
+		ui_notifier.set_ui_callback(ui_callback)
+		self.init_key_mapper(ui_notifier, command_map)
 
 
 # ------- parse_keys -------
@@ -115,8 +137,7 @@ def test__format_keys__ctrl_space() -> None:
 # ------- bind_key, unbind_key etc -------
 
 def test__bind_and_unbind_one_key() -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 
 	w.bind_key('a', 'foo')
 	assert w._default_command_map['a'] == 'foo'
@@ -125,8 +146,7 @@ def test__bind_and_unbind_one_key() -> None:
 	assert w._default_command_map['a'] == None
 
 def test__unbind_all() -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 
 	bound_keys = tuple(sut.iter_commandmap_keys(w._default_command_map))
 	assert len(bound_keys) > 0
@@ -137,8 +157,7 @@ def test__unbind_all() -> None:
 	assert len(bound_keys) == 0
 
 def test__bind_sequence() -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 	w.bind_key('ab', 'bar')
 
 	cmdmap = w._default_command_map['a']
@@ -149,8 +168,7 @@ def test__bind_sequence() -> None:
 	assert list(sut.iter_commandmap_items(cmdmap)) == [('b', 'bar')]
 
 def test__replace_command() -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 	assert w._default_command_map['left'] == urwid.CURSOR_LEFT
 	assert w._default_command_map['h'] == urwid.CURSOR_LEFT
 	assert w._default_command_map['right'] == urwid.CURSOR_RIGHT
@@ -162,8 +180,7 @@ def test__replace_command() -> None:
 	assert w._default_command_map['right'] == urwid.CURSOR_RIGHT
 
 def test__replace_command__key_sequence() -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 
 	w.bind_key('}', 'foo')
 	w.bind_key(']]', 'foo')
@@ -177,26 +194,25 @@ def test__replace_command__key_sequence() -> None:
 	assert w._default_command_map['ctrl w']['ctrl w'] == 'bar'
 
 
-# ------- save and load -------
+# ------- save and load command maps -------
 
-def test__save(create_test_dir: None) -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+def test__save1(create_test_dir: None) -> None:
+	w = SutKeyMapper(urwid.command_map.copy())
 	w.mapclear()
 	assert len(tuple(w.iter_commands_unsorted(w._default_command_map))) == 0
 
 	w.bind_key('a', 'foo')
 	w.bind_key('bcd', 'bar baz')
 
-	exporter = sut.CommandMapExporter(TestLogger(), w._default_command_map)
-	exporter.save(FN_CONFIG)
+	exporter = create_config_file(command_maps=w._default_command_map, commands=[sut.UrwidConfigFileArgparseCommand])
+	exporter.save_file(FN_CONFIG, comments=False)
 
 	with open(FN_CONFIG, 'rt') as f:
 		assert f.read() == '''\
 mapclear
 
-map bcd	'bar baz'
-map a	foo
+map bcd 'bar baz'
+map a foo
 '''
 
 def test__save__reference_command_map(create_test_dir: None) -> None:
@@ -223,36 +239,35 @@ def test__save__reference_command_map(create_test_dir: None) -> None:
 	menu.bind_key('u', 'unmount')
 
 	command_maps = dict(diff=diff._default_command_map, edit=edit._default_command_map, menu=menu._default_command_map)
-	exporter = sut.CommandMapExporter(TestLogger(), command_maps, refmap)
-	exporter.save(FN_CONFIG)
+	exporter = create_config_file(command_maps=command_maps, reference_command_map=refmap)
+	exporter.save_file(FN_CONFIG, commands=[sut.UrwidConfigFileArgparseCommand])
 
 	with open(FN_CONFIG, 'rt') as f:
 		assert f.read() == '''\
 mapclear
 
-map '<down>'	'cursor down'
-map '<left>'	'cursor left'
-map '<right>'	'cursor right'
-map '<up>'	'cursor up'
+map '<down>' 'cursor down'
+map '<left>' 'cursor left'
+map '<right>' 'cursor right'
+map '<up>' 'cursor up'
 
-map diff '<left>'	collapse
-map diff i	diff
-map diff '<right>'	expand
+map diff '<left>' collapse
+map diff i diff
+map diff '<right>' expand
 
-map edit '<ctrl k>'	'delete after cursor'
-map edit '<ctrl u>'	'delete before cursor'
+map edit '<ctrl k>' 'delete after cursor'
+map edit '<ctrl u>' 'delete before cursor'
 unmap edit '<up>'
 unmap edit '<down>'
 
-map menu m	mount
-map menu u	unmount
+map menu m mount
+map menu u unmount
 '''
 
 
 
 def test__load__map(create_test_dir: None) -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 	assert w._default_command_map['left'] == urwid.CURSOR_LEFT
 
 	with open(FN_CONFIG, 'wt') as f:
@@ -261,8 +276,8 @@ map a foo
 map bcd 'bar baz'
 ''')
 
-	exporter = sut.CommandMapExporter(TestLogger(), w._default_command_map)
-	exporter.load(FN_CONFIG)
+	exporter = create_config_file(command_maps=w._default_command_map)
+	exporter.load_file(FN_CONFIG)
 	assert w._default_command_map['a'] == 'foo'
 	assert isinstance(w._default_command_map['b'], sut.SubCommandMap)
 	assert isinstance(w._default_command_map['b']['c'], sut.SubCommandMap)
@@ -290,8 +305,8 @@ map  a,b  foo     'bar baz'
 ''')
 
 	command_maps = dict(a=a._default_command_map, b=b._default_command_map, c=c._default_command_map)
-	exporter = sut.CommandMapExporter(TestLogger(), command_maps)
-	exporter.load(FN_CONFIG)
+	exporter = create_config_file(command_maps=command_maps)
+	exporter.load_file(FN_CONFIG)
 	assert a._default_command_map['left'] == 'foo'
 	assert b._default_command_map['left'] == urwid.CURSOR_LEFT
 	assert c._default_command_map['left'] == urwid.CURSOR_LEFT
@@ -305,16 +320,15 @@ map  a,b  foo     'bar baz'
 	assert c._default_command_map['right'] == urwid.CURSOR_RIGHT
 
 def test__load__unmap(create_test_dir: None) -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 
 	assert w._default_command_map['left'] == urwid.CURSOR_LEFT
 
 	with open(FN_CONFIG, 'wt') as f:
 		f.write('unmap <left>')
 
-	exporter = sut.CommandMapExporter(TestLogger(), w._default_command_map)
-	exporter.load(FN_CONFIG)
+	exporter = create_config_file(command_maps=w._default_command_map)
+	exporter.load_file(FN_CONFIG)
 	assert w._default_command_map['left'] is None
 
 def test__load__unmap_specific(create_test_dir: None) -> None:
@@ -332,18 +346,17 @@ def test__load__unmap_specific(create_test_dir: None) -> None:
 		f.write('unmap b,c <left>')
 
 	command_maps = dict(a=a._default_command_map, b=b._default_command_map, c=c._default_command_map)
-	exporter = sut.CommandMapExporter(TestLogger(), command_maps)
-	exporter.load(FN_CONFIG)
+	exporter = create_config_file(command_maps=command_maps)
+	exporter.load_file(FN_CONFIG)
 	assert a._default_command_map['left'] == urwid.CURSOR_LEFT
-	assert b._default_command_map['left'] is None  # mypy does not understand that this value has been changed by exporter.load
+	assert b._default_command_map['left'] is None  # mypy does not understand that this value has been changed by exporter.load_file
 	assert c._default_command_map['left'] is None  # type: ignore [unreachable]
 	assert a._default_command_map['right'] == urwid.CURSOR_RIGHT
 	assert b._default_command_map['right'] == urwid.CURSOR_RIGHT
 	assert c._default_command_map['right'] == urwid.CURSOR_RIGHT
 
 def test__load__mapclear(create_test_dir: None) -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 
 	assert w._default_command_map['left'] == urwid.CURSOR_LEFT
 	assert len(tuple(w.iter_commands_unsorted(w._default_command_map))) > 0
@@ -351,9 +364,9 @@ def test__load__mapclear(create_test_dir: None) -> None:
 	with open(FN_CONFIG, 'wt') as f:
 		f.write('mapclear')
 
-	exporter = sut.CommandMapExporter(TestLogger(), w._default_command_map)
-	exporter.load(FN_CONFIG)
-	assert w._default_command_map['left'] is None  # mypy does not understand that this value has been changed by exporter.load
+	exporter = create_config_file(command_maps=w._default_command_map)
+	exporter.load_file(FN_CONFIG)
+	assert w._default_command_map['left'] is None  # mypy does not understand that this value has been changed by exporter.load_file
 	assert len(tuple(w.iter_commands_unsorted(w._default_command_map))) == 0  # type: ignore [unreachable]
 
 def test__load__mapclear_specific(create_test_dir: None) -> None:
@@ -368,8 +381,8 @@ def test__load__mapclear_specific(create_test_dir: None) -> None:
 		f.write('mapclear c345,a123')
 
 	command_maps = dict(a123=a._default_command_map, b234=b._default_command_map, c345=c._default_command_map)
-	exporter = sut.CommandMapExporter(TestLogger(), command_maps)
-	exporter.load(FN_CONFIG)
+	exporter = create_config_file(command_maps=command_maps)
+	exporter.load_file(FN_CONFIG)
 	for m in (a, c):
 		assert m._default_command_map['left'] is None
 		assert len(tuple(m.iter_commands_unsorted(m._default_command_map))) == 0
@@ -381,33 +394,31 @@ def test__load__mapclear_specific(create_test_dir: None) -> None:
 
 
 def test__save_and_load_default_command(create_test_dir: None) -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 
 	assert w._default_command_map['left'] == urwid.CURSOR_LEFT
 
-	exporter = sut.CommandMapExporter(TestLogger(), w._default_command_map)
-	exporter.save(FN_CONFIG)
+	exporter = create_config_file(command_maps=w._default_command_map)
+	exporter.save_file(FN_CONFIG)
 	assert w._default_command_map['left'] == urwid.CURSOR_LEFT
 
 	w.mapclear()
 	assert w._default_command_map['left'] is None
 
-	exporter.load(FN_CONFIG)  # type: ignore [unreachable]
+	exporter.load_file(FN_CONFIG)  # type: ignore [unreachable]
 	assert w._default_command_map['left'] == urwid.CURSOR_LEFT
 
 def test__save_and_load_undo_bind_key(create_test_dir: None) -> None:
-	w = sut.KeyMapper()
-	w.init_key_mapper(TestLogger(), urwid.command_map.copy())
+	w = SutKeyMapper(urwid.command_map.copy())
 
-	exporter = sut.CommandMapExporter(TestLogger(), w._default_command_map)
-	exporter.save(FN_CONFIG)
+	exporter = create_config_file(command_maps=w._default_command_map)
+	exporter.save_file(FN_CONFIG)
 	assert w._default_command_map['a'] is None
 
 	w.bind_key('a', 'foo')
 	assert w._default_command_map['a'] == 'foo'
 
-	exporter.load(FN_CONFIG)
+	exporter.load_file(FN_CONFIG)
 	assert w._default_command_map['a'] is None
 
 def test__save_and_load_multiple_command_maps(create_test_dir: None) -> None:
@@ -437,8 +448,8 @@ def test__save_and_load_multiple_command_maps(create_test_dir: None) -> None:
 	assert a._default_command_map['right'] == urwid.CURSOR_RIGHT  # type: ignore [unreachable]
 
 	command_maps = dict(a123=a._default_command_map, b234=b._default_command_map, c345=c._default_command_map)
-	exporter = sut.CommandMapExporter(TestLogger(), command_maps)
-	exporter.save(FN_CONFIG)
+	exporter = create_config_file(command_maps=command_maps)
+	exporter.save_file(FN_CONFIG)
 	assert a._default_command_map['left'] == 'foo a'
 	assert a._default_command_map['right'] == urwid.CURSOR_RIGHT
 	assert a._default_command_map['f'] is None
@@ -459,7 +470,7 @@ def test__save_and_load_multiple_command_maps(create_test_dir: None) -> None:
 	c.bind_key('<right>', urwid.CURSOR_RIGHT)
 	assert c._default_command_map['right'] == urwid.CURSOR_RIGHT
 
-	exporter.load(FN_CONFIG)
+	exporter.load_file(FN_CONFIG)
 	assert a._default_command_map['left'] == 'foo a'
 	assert a._default_command_map['right'] == urwid.CURSOR_RIGHT
 	assert a._default_command_map['f'] is None
@@ -471,6 +482,64 @@ def test__save_and_load_multiple_command_maps(create_test_dir: None) -> None:
 	assert c._default_command_map['f'] is None
 
 
+# ------- save and load config -------
+
+def test_save_and_load_help_item(reset_config: None, create_test_dir: None) -> None:
+	CURSOR_LEFT = 'cursor left'
+	CURSOR_RIGHT = 'cursor right'
+
+	class MyTestClass:
+		help_content = Config('fmt', [
+			sut.HelpItem(CURSOR_LEFT, 'left'),
+			sut.HelpItem(CURSOR_RIGHT, 'right'),
+		])
+
+	t = MyTestClass()
+	exporter = create_config_file()
+	exporter.save_file(FN_CONFIG)
+	assert t.help_content == [
+		sut.HelpItem(CURSOR_LEFT, 'left'),
+		sut.HelpItem(CURSOR_RIGHT, 'right'),
+	]
+
+	t.help_content.clear()
+	assert t.help_content == []
+
+	exporter.load_file(FN_CONFIG)
+	assert t.help_content == [
+		sut.HelpItem(CURSOR_LEFT, 'left'),
+		sut.HelpItem(CURSOR_RIGHT, 'right'),
+	]
+
+def test_save_and_load_help_item_with_alternative_commands(reset_config: None, create_test_dir: None) -> None:
+	QUIT = ['quit', 'quit --ask', 'quit --ask-if-long-startup']
+	HELP = 'help'
+
+	class MyTestClass:
+		help_content = Config('fmt', [
+			sut.HelpItem(QUIT, 'quit'),
+			sut.HelpItem(HELP, 'help'),
+		])
+
+	t = MyTestClass()
+	exporter = create_config_file()
+	exporter.save_file(FN_CONFIG)
+	assert t.help_content == [
+		sut.HelpItem(QUIT, 'quit'),
+		sut.HelpItem(HELP, 'help'),
+	]
+
+	t.help_content.clear()
+	assert t.help_content == []
+
+	exporter.load_file(FN_CONFIG)
+	assert t.help_content == [
+		sut.HelpItem(QUIT, 'quit'),
+		sut.HelpItem(HELP, 'help'),
+	]
+
+
+
 # ------- HelpBar -------
 
 def test_help_bar_is_key_mapped__true_single_command() -> None:
